public class KISpieler {

	public Spiel spiel;
	public boolean rot;

	public KISpieler(GUI_NB gui, boolean newRot) {
		this.spiel = gui.spiel.klonen(gui.spiel);
		rot = newRot;
	}

	public int move() {
		return minimax(rot, 4)[1];
	}

	private int[] minimax(boolean istRot, int tiefe) {
		int[][] moeglichkeiten=spiel.moeglichePositionen();
		
		int besterZug=-1;
		int besteBew=(istRot==this.rot ? Integer.MIN_VALUE : Integer.MAX_VALUE);
		int aktBew;
		
		if(spiel.sieger()!='N'||tiefe==0||spiel.Unentschieden()){
			besteBew=bewertung();
		}
		else{
			for(int[] tmp:moeglichkeiten){
				spiel.zug(istRot, tmp[1]+1);
				aktBew=minimax(!istRot,tiefe-1)[0];
				if(istRot==this.rot){
					if(aktBew>besteBew){
						besteBew=aktBew;
						besterZug=tmp[1]+1;
					}
				}
				else{
					if(aktBew<besteBew){
						besteBew=aktBew;
						besterZug=tmp[1]+1;
					}
				}
				spiel.SteinEntfernen(tmp[0], tmp[1]);
			}
		}
		
		return new int[]{besteBew,besterZug};
	}

	private int bewertung() {
		int bewertung = 0;

		for (int y = 0; y < 7; ++y) {
			for (int x = 0; x < 7; ++x) {
				Spielstein cursor = spiel.getSteine()[y][x];
				if (cursor != null) {
					int zwischenErg = 0;
					int nachR = 3, nachL = 3, nachO = 3, nachU = 3, nachOR, nachOL, nachUR, nachUL, reiheHori = 0,
							reiheVerti = 0, reiheORUL = 0, reiheUROL = 0, laengsteReihe;
					if (x > 3 && x < 7) {
						nachR = 3 - (x % 4);
					}
					if (x >= 0 && x < 4) {
						nachL = x;
					}
					if (y > 3 && y < 7) {
						nachU = 3 - (y % 4);
					}
					if (y >= 0 && y < 4) {
						nachO = y;
					}
					nachOR = Math.min(nachO, nachR);
					nachOL = Math.min(nachO, nachL);
					nachUR = Math.min(nachU, nachR);
					nachUL = Math.min(nachU, nachL);

					for (int tmp = 0; tmp <= 3; ++tmp) {
						if (nachR - tmp > 0) {
							if (spiel.getSteine()[y][x + tmp] != null) {
								if (spiel.getSteine()[y][x + tmp].rot != cursor.rot) {
									nachR = 0;
								}
								else{
									--nachL;
									++reiheHori;
								}
							} else {
								nachR = 0;
							}
						}
						if (nachL - tmp > 0) {
							if (spiel.getSteine()[y][x - tmp] != null) {
								if (spiel.getSteine()[y][x - tmp].rot != cursor.rot) {
									nachL = 0;
								}
								else {
									--nachR;
									++reiheHori;
								}
							} else {
								nachL = 0;
							}
						}
						if (nachU - tmp > 0) {
							if (spiel.getSteine()[y+tmp][x] != null) {
								if (spiel.getSteine()[y+tmp][x].rot != cursor.rot) {
									nachU = 0;
								}
								else {
									--nachO;
									++reiheVerti;
								}
							} else {
								nachU = 0;
							}
						}
						if(nachO-tmp>0){
							if(spiel.getSteine()[y-tmp][x]!=null){
								if(spiel.getSteine()[y-tmp][x].rot!=cursor.rot){
									nachO=0;
								}
								else{
									--nachU;
									++reiheVerti;
								}
							}else{
								nachO=0;
							}
						}
						if(nachOR-tmp>0){
							if(spiel.getSteine()[y-tmp][x+tmp]!=null){
								if(spiel.getSteine()[y-tmp][x+tmp].rot!=cursor.rot){
									nachOR=0;
								}
								else{
									--nachUL;
									++reiheORUL;
								}
							}
							else{
								nachOR=0;
							}
						}
						if(nachUL-tmp>0){
							if(spiel.getSteine()[y+tmp][x-tmp]!=null){
								if(spiel.getSteine()[y+tmp][x-tmp].rot!=cursor.rot){
									nachUL=0;
								}
								else{
									--nachOR;
									++reiheORUL;
								}
							}else{
								nachUL=0;
							}
						}
						if(nachOL-tmp>0){
							if(spiel.getSteine()[y-tmp][x-tmp]!=null){
								if(spiel.getSteine()[y-tmp][x-tmp].rot!=cursor.rot){
									nachOL=0;
								}
								else{
									--nachUR;
									++reiheUROL;
								}
							}
							else{
								nachOL=0;
							}
						}
						if(nachUR-tmp>0){
							if(spiel.getSteine()[y+tmp][x+tmp]!=null){
								if(spiel.getSteine()[y+tmp][x+tmp].rot!=cursor.rot){
									nachUR=0;
								}
								else{
									--nachOL;
									++reiheUROL;
								}
							}else{
								nachUR=0;
							}
						}
					}
					laengsteReihe = Math.max(reiheHori, Math.max(reiheVerti, Math.max(reiheORUL, reiheUROL)));
					if(laengsteReihe==4){
						return (cursor.rot==this.rot ? Integer.MAX_VALUE : Integer.MIN_VALUE);
					}
					zwischenErg = potenz(10, laengsteReihe)*(cursor.rot==this.rot ? 1 : (-1));
					if (zwischenErg > bewertung) {
						bewertung = zwischenErg;
					}
				}
			}
		}

		return bewertung;
	}

	// x^y
	public int potenz(int x, int y) {
		if (y <= 0) {
			return 1;
		}
		for (; y > 0; --y) {
			x *= x;
		}
		return x;
	}
}